/*
JMP Clinical Analytical Process

Name: ClusterSubjectsStudySite
Authors: Richard C. Zink, SAS Institute Inc.

Description: 

Performs Cluster subjects within study sites.
	
*/

/*
-----------------------------------------------------------
Copyright (C) 2012 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName ClusterSubjectsScript ClusterSubjectsStudySite_OutData OutPath 
        ServerOutPath PopulationSet MissMax FindDistFl InterDistFl EventDistFl WhereStmt WhereForReport DelMissFl HierClustMethod AgeSexFl;

*ProcessBody;

%macro ClusterSubjectsStudySite;

*initialization and argument checking;

%let ProcessName = ClusterSubjectsStudySite; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;
%let WhereForReport=%nrbquote(&WhereStmt);
%if &AgeSexFl = No and &EventDistFl = No and &InterDistFl = No and &FindDistFl = No %then %do;
	%put ERROR: At least one domain type must be selected for analysis.;
	%let exiterror = 1;
	%goto exit;
%end;

%let trCountry = %nrbquote(%LocaleGet(Country_Text));
%let trNotProvided = %nrbquote(%LocaleGet(NotProvided_Text));

%if %sysfunc(findw(%upcase(&ADSLVARS), AGE, ' ')) %then %let ageflag = age; 
%else %let ageflag = ;

%if %sysfunc(findw(%upcase(&ADSLVARS), SEX, ' ')) %then %let sexflag = sex;
%else %let sexflag = ;

%let _tid_ = %sysfunc(open(&in_adsl));
%let vn = %sysfunc(varnum(&_tid_,USUBJID));
%let USubLen = %sysfunc(varlength(&_tid_,&vn));
%let _tid_ = %sysfunc(close(&_tid_));

%if ^%length(&findingsdomains) and %upcase(&FindDistFl) = YES %then %do;
	%put ERROR: No Findings domains are available for the current study.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%length(&interventionsdomains) and %upcase(&InterDistFl) = YES %then %do;
	%put ERROR: No Interventions domains are available for the current study.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%length(&eventsdomains) and %upcase(&EventDistFl) = YES %then %do;
	%put ERROR: No Events domains are available for the current study.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM, ' ')) %then %do;
	%put ERROR: ARM is not available in either DM, ADDM or ADSL.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID, ' ')) %then %do;
	%put ERROR: SITEID is not available in either DM, ADDM or ADSL.;
	%let exiterror = 1;
	%goto exit;
%end;

%let _tid_ = %sysfunc(open(&in_adsl));
%let usubjvn = %sysfunc(varnum(&_tid_,USUBJID));
%let SubLab = %sysfunc(varlabel(&_tid_,&usubjvn));
%let sitevn = %sysfunc(varnum(&_tid_,SITEID));
%let SiteLab = %sysfunc(varlabel(&_tid_,&sitevn));
%let _tid_ = %sysfunc(close(&_tid_));  
    	
%if %index(%upcase(&in_adsl), ADSL) and %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY, ' ')) = 0 and %sysfunc(findw(%upcase(&DMVARS), COUNTRY, ' ')) > 0 %then %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm &ageflag &sexflag);
		by usubjid;
	run;

	proc sort data = &in_dm out = dm;
		by usubjid;
	run;

	data subject;
		merge subject(in = ina) dm(keep = usubjid country);
		by usubjid;
		if ina;
	run;
%end;
%else %if %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY, ' ')) %then %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm country &ageflag &sexflag);
		by usubjid;
	run;
%end;
%else %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm &ageflag &sexflag);
		by usubjid;
	run;
	
	data subject;
	   length Country $ 12;
	   set subject;
	   country = "&trNotProvided";
	   label country = "&trCountry";
	run;
%end;

proc sql noprint;
   create table subpairs as
      select aa.usubjid, bb.usubjid2, aa.siteid
      from subject as aa, subject(rename = (usubjid = USUBJID2 siteid = siteid2)) as bb 	
      where aa.usubjid < bb.usubjid2 and siteid = siteid2	
      order by aa.usubjid, bb.usubjid2;
quit;

%let availabledomains = ;

%let trUnscheduled = %nrbquote(%LocaleGet(Unscheduled_Text));

*** create by lists for each domain ***;
%if &FindDistFl = Yes %then %do;
	%let i = 1;
	%do %while(%length(%scan(&findingsdomains,&i,' ')));
		%let temp = %scan(&findingsdomains,&i,' ');

		%if %sysfunc(indexw(&&&temp.VARS, VISITNUM, ' ')) and %sysfunc(indexw(&&&temp.VARS, &temp.STRESN, ' ')) %then %do;
			%let availabledomains = &availabledomains &temp;
			%if %sysfunc(indexw(&&&temp.VARS, &temp.TPTNUM, ' ')) %then %let tpt = &temp.tptnum;
			%else %let tpt =;

			%if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %let visit = visit;
			%else %let visit = ;

			proc sort data = &&in_&temp(where = (not missing (&temp.STRESN))) out = &temp(keep = usubjid &temp.testcd &temp.STRESN VISITNUM &tpt &visit);
				by usubjid &temp.testcd visitnum &tpt;
			run;

			*** Delete unscheduled records ***;
			%if %sysfunc(indexw(&&&temp.VARS, VISIT, ' ')) %then %do;
				data &temp(drop = visit);
					set &temp;
					if index(upcase(visit), "UNSCHEDULED") or kindex(kupcase(visit), "%kupcase(&trUnscheduled)") then delete;
				run;
			%end;

			ods exclude all;
			ods noresults;
			proc means noprint nway missing data = &temp;
				class usubjid &temp.testcd visitnum &tpt;
				var &temp.STRESN;
				output out = &temp(drop = _type_ _freq_) mean = &temp.STRESN;
			run;
			ods select all;
			ods results;

			data &temp;
				set &temp;
				%if %length(&tpt) %then %do;
				   if missing(&tpt) then level = cats(&temp.testcd,"_","V",visitnum);         
				   else level = cats(&temp.testcd,"_","V",visitnum,"_","T",&tpt);         
				%end;
				%else %do;
					level = cats(&temp.testcd,"_","V",visitnum);
				%end;
			run;

			proc sort data = &temp out = &temp;
				by usubjid level;
			run;

			proc transpose data = &temp out = &temp(drop = _name_ _label_);
				by usubjid;
				var &temp.STRESN;
				id level;
				idlabel level;
			run;              
		%end;
		%let i = %eval(&i+1);
	%end;
%end;

%macro INTEVENT(list = );
	%let i = 1;
	%do %while(%length(%scan(&list,&i,' ')));
		%let temp = %scan(&list,&i,' ');

		%if %sysfunc(indexw(&&&temp.VARS, &temp.DECOD, ' ')) %then %do;
			%let availabledomains = &availabledomains &temp;

			proc sort data = &&in_&temp(where = (not missing (&temp.DECOD))) out = &temp;
				by usubjid &temp.DECOD;
			run;
         
			*** Delete prespecified not occuring records ***;
			data &temp(keep = usubjid &temp.DECOD count);
				set &temp;
				count = 1;
				%PRESPECIFY(dom = &temp);
			run;

         proc freq noprint data = &temp;
            tables &temp.DECOD / out = outname(drop = count percent);
         run;
                  
         data outname;
            length &temp.short $ 8;
            set outname;
            &temp.short = cats("&temp",_n_);   
         run;

         proc sort data = &temp;
            by usubjid &temp.DECOD;
			run;

			ods exclude all;
			ods noresults;
			proc means noprint nway data = &temp;
				class usubjid &temp.DECOD;
				var count;
				output out = &temp(drop = _type_ _freq_) n = count;
			run;
			ods select all;
			ods results;

			proc sql noprint;      
				create table events as
					select distinct &temp.DECOD
					from &temp
					order by &temp.DECOD;		
			quit;

			proc sql;
				create table events1 as
				select aa.usubjid, bb.&temp.DECOD
				from subject as aa, events as bb
				order by aa.usubjid, bb.&temp.DECOD;
			quit;

			data &temp;
			  merge &temp(in = ina) events1(in = inb);
			  by usubjid &temp.DECOD;
			  if inb and not ina then count = 0;		  
			run;

         proc sort data = &temp;
            by &temp.decod;
         run;
           
         data &temp;
            merge &temp outname;
            by &temp.decod;
         run;

			proc datasets nolist;
				delete events events1 outname;
			quit;
			
         proc sort data = &temp;
            by usubjid &temp.decod;
         run;			
           
			proc transpose data = &temp out = &temp(drop = _name_ _label_);
				by usubjid;
				var count;
				id &temp.short; 
				idlabel &temp.DECOD;
			run;   
		%end;

		%let i = %eval(&i+1);
	%end;
%mend INTEVENT;

%if &EventDistFl = Yes %then %do;
	%INTEVENT(list = &eventsdomains);
%end;

%if &InterDistFl = Yes %then %do;
	%INTEVENT(list = &interventionsdomains); 
%end;

%let trMaleGender = %nrbquote(%LocaleGet(MaleGender_Text));
%let trM = %nrbquote(%LocaleGet(M_Text));
%let trF = %nrbquote(%LocaleGet(F_Text));

data all;
   merge subject(in = ina) &availabledomains;
   by usubjid;
   if ina;
   %if %length(&sexflag) %then %do;
      if sex = "M" or sex = "&trM" then MALE = 1;
      else if sex = "F" or sex = "&trF" then MALE = 0;
      drop sex;
      label male = "&trMaleGender";
   %end;
	%if &AgeSexFl = No %then %do;
		%if %length(&ageflag) %then %do;
      	drop age;
		%end;
		%if %length(&sexflag) %then %do;
			drop male;
		%end;
	%end;   
run;
%NObsVars(all);
%let TotalSubs = &nobs;

proc means nmiss noprint data = all;
	output out = nmiss(drop = _type_ _freq_) nmiss = / autoname;
run;

proc transpose data = nmiss out = nmiss;
run;

%let trVariable = %nrbquote(%LocaleGet(Variable_Text));
%let trLabel = %nrbquote(%LocaleGet(Label_Text));
%let trNOSMV = %nrbquote(%LocaleGet(NumberofSubjectsMissingValues_Text));
%let trPOSMV = %nrbquote(%LocaleGet(PercentageofSubjectsMissingValues_Text));
%let trMissFlag = %nrbquote(%LocaleGet(MissingFlag_Text));

data nmiss;
   set nmiss(rename = (col1 = nummiss)) end = eof;
   _name_ = substr(_name_, 1, length(_name_) - 6);
   pctmiss = round(nummiss / &TotalSubs * 100, 0.01);
   %if %upcase(&DelMissFl) = YES %then %do; 
   	if pctmiss > 0 then missflag = 2;
   %end;
   %else %do;
   	if pctmiss >= &MissMax then missflag = 2;
   %end;
   else if pctmiss ne 0 then missflag = 1;
   else if pctmiss = 0 then missflag = 0;
   if eof then call symput("TotalVars", trim(left(_n_)));
   label _name_ = "&trVariable" _label_ = "&trLabel" nummiss = "&trNOSMV" pctmiss = "&trPOSMV" missflag = "&trMissFlag";
run;

*** degree of freeom is number of variables contributing to maha scores;
%let DegFree = 0;
data _null_;
   set nmiss(where = (missflag in (0,1))) end = eof;
   if eof then call symput("DegFree", trim(left(_n_)));
run;

%let RemovedVars = ;
data _null_;
   set nmiss(where = (missflag = 2)) end = eof;
   if eof then call symput("RemovedVars", trim(left(_n_)));
run;

%if ^%length(&RemovedVars) %then %let RemovedVars = 0;
%if ^%eval(&TotalVars - &RemovedVars) %then %do;
	%put ERROR: No variables remain for analysis after deleting variables with at least &MissMax percent missing data.;
	%let exiterror = 1;
	%goto exit;
%end;
%let OtherVars = %eval(&TotalVars - &RemovedVars - &DegFree);

%let MissMaxVars = ;
%let MissMaxLabels = ;
%let MissLabels = ;
proc sql noprint;
	select _name_ 
		into :MissMaxVars separated by ' '
		from nmiss
		where missflag = 2
		order by _name_;
/*	select _label_ 
		into :MissMaxLabels separated by ' '
		from nmiss
		where missflag = 2
		order by _name_;		
	select _label_ 
		into :MissLabels separated by ' '
		from nmiss
		where missflag = 1
		order by _name_; */
quit;

data all;
   set all(drop = &MissMaxVars);
   subject = cats("S",_n_);
run;

%let includeds =;

%let ClusterSubjects_OutData_sum = &ClusterSubjectsStudySite_OutData;	   
%PrepOutDS(ClusterSubjects_OutData_sum,OutDataName_sum, clustsubs_summary);
%let includeds = &includeds &OutDataName_sum..sas7bdat,;
	
data &ClusterSubjects_OutData_sum;
   set all(drop = subject) end = eof;
run;

%let SiteList = ;

proc sql noprint;
	select distinct siteid, count(siteid) as count 
		into :SiteList separated by ' '
		from all
		group by siteid
		having calculated count >= 2
		order by siteid;
quit;

proc means var nway noprint data = all;
	class siteid;
	output out = novar(drop = _type_ _freq_) var = / autoname;
run;

proc transpose data = novar out = novar;
   by siteid;
run;

proc sql noprint;
   create table SiteVarCountList as
	select distinct siteid, count(siteid) as countvars 
		from novar
		group by siteid
		order by siteid;
quit;

data novar;
   set novar(rename = (col1 = var) where = (var in (.,0))) end = eof;
   _name_ = substr(_name_, 1, length(_name_) - 4);
run;

proc sql noprint;
   create table SiteNoVarCountList as
	select distinct siteid, count(siteid) as countnovar
		from novar
		group by siteid
		order by siteid;
quit;

data SiteVarCountList;
    merge SiteVarCountList SiteNoVarCountList;
    by siteid;
run;

proc datasets nolist;
   delete SiteNoVarCountList;
quit;

%let SitesNoVariability =;
%let firstflag = 0;

%let trDistance = %nrbquote(%LocaleGet(Distance_Text));

%let i = 1;
%do %while(%length(%scan(&SiteList,&i,' ')));
	%let tempsite = %scan(%upcase(&SiteList),&i," ");	
	%let SiteConstantVars = ;	
	proc sql noprint;
		select _name_ 
			into :SiteConstantVars separated by ' '
			from novar
			where siteid = "&tempsite"
			order by _name_;
	quit;

   %let varok = 1;
   data _null_;
      set SiteVarCountList(where = (siteid = "&tempsite"));
      if countvars = countnovar then do;
      	call symput("varok",trim(left(0)));
      end;
   run;
   
   %if ^&varok %then %let SitesNoVariability = &SitesNoVariability &tempsite;
  
   %if &varok %then %do;   
		proc distance data = all(drop = &SiteConstantVars) shape = square method = euclid out = distance;
			var interval(_numeric_ / std = std);
			copy usubjid siteid arm country;
			id subject;
			where siteid = "&tempsite";
		run;

		proc sql noprint;
			select usubjid 
				into :SiteClusterVars separated by ' '
				from distance
				order by usubjid;
		quit;

		%let SiteClusterVars&tempsite = ;

		%let j = 1;
		%do %while(%length(%scan(&SiteClusterVars,&j,' ')));
			%let temp = %scan(%upcase(&SiteClusterVars),&j," ");      
			%let SiteClusterVars&tempsite = &&SiteClusterVars&tempsite %bquote(:Name(""&temp"")),;
			%let j = %eval(&j+1);
		%end;

		proc transpose data = distance out = distance;
			by usubjid subject siteid arm country;
		run;

		proc sql noprint;
			create table siteids as
				select distinct subnum, usubjid2
				from all(rename = (subject = subnum usubjid = USUBJID2))
				order by subnum;		
			create table distance2 as
				select * 
				from distance as aa left join siteids as bb
				on aa._name_ = bb.subnum
				order by usubjid, usubjid2;
		quit;

		proc datasets nolist;
			delete siteids distance;
			change distance2 = distance;
		quit;

		proc sql noprint;
			create table distancepairs(drop = subnum subject _name_ rename = (col1 = Distance)) as
				select *, min(col1) as mindist
				from subpairs(where = (siteid = "&tempsite")) natural left join distance
				order by usubjid, usubjid2;		
		quit;

		data mindisttemp(drop = mindist);
			set distancepairs(where = (distance = mindist));
			if _n_ = 1;
		run;

		%if ^&firstflag %then %do;
			data mindist;
				set mindisttemp;
			run;
		%end;
		%else %do;   
			proc append force base = mindist data = mindisttemp;
			run;
		%end;

		data distancepairs;
			set distancepairs(drop = mindist);
			label distance = "&trDistance" usubjid = "&SubLab" usubjid2 = "&SubLab 2" siteid = "&SiteLab";
		run;

		%if ^&firstflag %then %do;
			data alldistancepairs;
				set distancepairs;
			run;
		%end;
		%else %do;   
			proc append force base = alldistancepairs data = distancepairs;
			run;
		%end;

		%if ^&firstflag %then %let firstflag = 1;

		%if &i = 1 %then %do;   
			%let SUBJECTLabel2 = USUBJID2;
			%CheckLabel(distancepairs,SUBJECTLabel2);
		%end;

		proc transpose data = distance out = distance(drop = _name_);
			var col1;
			by usubjid siteid arm country;
			id _name_;
			idlabel usubjid2;
		run;

		%if %nrbquote(&SYSENCODING) = wlatin1 %then %do;
			%let ClusterSubjects_OutData_&tempsite = &ClusterSubjectsStudySite_OutData;	   
			%let ClusterSubjects_OutDataP_&tempsite = &ClusterSubjectsStudySite_OutData;	   

			%PrepOutDS(ClusterSubjects_OutData_&tempsite,OutDataName_&tempsite, clustsubs_site, _&tempsite);
			%PrepOutDS(ClusterSubjects_OutDataP_&tempsite,OutDataNameP_&tempsite, clustsubspairs_site, _&tempsite);

			%let includeds = &includeds &&&OutDataName_&tempsite...sas7bdat, &&&OutDataNameP_&tempsite...sas7bdat,;
	
			data &&&ClusterSubjects_OutData_&tempsite;
					set distance;
				run;

			data &&&ClusterSubjects_OutDataP_&tempsite;
					set distancepairs;
				run;
		 %end;
		 %else %do;
			%let ClusterSubjects_OutData_&i = &ClusterSubjectsStudySite_OutData;	   
			%let ClusterSubjects_OutDataP_&i = &ClusterSubjectsStudySite_OutData;	   

			%PrepOutDS(ClusterSubjects_OutData_&i,OutDataName_&i, clustsubs_site, _&tempsite);
			%PrepOutDS(ClusterSubjects_OutDataP_&i,OutDataNameP_&i, clustsubspairs_site, _&tempsite);

			%let includeds = &includeds &&&OutDataName_&i...sas7bdat, &&&OutDataNameP_&i...sas7bdat,;

			data &&&ClusterSubjects_OutData_&i;
				set distance;
			run;

			data &&&ClusterSubjects_OutDataP_&i;
				set distancepairs;
			run;
		%end;

		proc datasets nolist;
			delete distance distancepairs mindisttemp;	 
		quit;
	%end;	
   %let i = %eval(&i+1);
%end; 

%if %sysfunc(exist(mindist)) %then %do;
	proc sort data = mindist;
		by distance;
	run;

	%let ClusterSubjects_OutData_min = &ClusterSubjectsStudySite_OutData;	   
	%PrepOutDS(ClusterSubjects_OutData_min,OutDataName_min, clustsubs_within_mindist);
	%let includeds = &includeds &OutDataName_min..sas7bdat,;

	data &ClusterSubjects_OutData_min;
		set mindist end = eof;
		if _n_ = 1 then call symput("MinDistSite", trim(siteid));
		label distance = "&trDistance" usubjid = "&SubLab" usubjid2 = "&SubLab 2" siteid = "&SiteLab";
		color = 1;
	run;
%end;

%if %sysfunc(exist(alldistancepairs)) %then %do;
	%let ClusterSubjects_OutData_all = &ClusterSubjectsStudySite_OutData;	   
	%PrepOutDS(ClusterSubjects_OutData_all,OutDataName_all, clustsubs_within_alldist);
	%let includeds = &includeds &OutDataName_all..sas7bdat,;

	data &ClusterSubjects_OutData_all;
		set alldistancepairs end = eof;
		color = 1;
	run;

	proc datasets nolist;
		delete alldistancepairs;
	quit;
%end;

%let ARMLabel = ARM;
%CheckLabel(all,ARMLabel);
%let SUBJECTLabel = USUBJID;
%CheckLabel(all,SUBJECTLabel);
%let COUNTRYLabel = COUNTRY;
%CheckLabel(all,COUNTRYLabel);
%let SITEIhlabel = SITEID;
%CheckLabel(all,SITEIhlabel);

proc datasets nolist;
   delete all;
quit;		

* create JSL file;
%PrepOutFile(ClusterSubjectsScript,ClusterSubjectsScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &ClusterSubjectsScript;
%let FullAPName=%str(Cluster Subjects Within Study Site);

* put JSL statements;
data _null_;
   file "&ClusterSubjectsScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "Names Default to Here(1);";
	put "Here:DataList = {};";
	put "Here:TabButtonList = {};";
	put "Here:TabCodeList = {};";
	put "Here:TabAfterCodeList = {};";
	put "Here:TabOpenList = {};";
	put "Here:CreateReportList = {};";
	put "Here: TabsSectionFlag = 1;";

  	put "Here:Select=Function({},{default local},";
	put "ctable = Current Data Table();";
	put "If(Is Empty(ctable),";
	put "   New Window(locale:get(""JMPClinicalMessage_Text""), showmenu(0),showtoolbars(0),";
	put "      tb = TextBox(locale:get(""ErrorToRunOpenInFocus_Text"")),";
	put "   	  tb << Font Color(3)";
	put "   );";
	put "   Throw()";
	put ");";
	put "subname = ""&SUBJECTLabel"";";
	put "subname2 = ""&SUBJECTLabel 2"";";
	put "foundit = 1;";
	put "foundit2 = 1;";
	put "try(scol = Column(subname),foundit = 0);";
	put "try(scol2 = Column(subname2),foundit2 = 0);";
	put "selsub = {};";
	put "nsel = 0;";
	put "	for each row(";
	put "		if(selected(rowstate())==1,";
	put "			if(!Contains(selsub,scol()[]),";
	put "				nsel++;";
	put "				InsertInto(selsub,scol()[])";
	put "			);";
	put "			if(foundit2 == 1,";
	put "				if(!Contains(selsub,scol2()[]),";
	put "					nsel++;";
	put "					InsertInto(selsub,scol2()[])";
	put "				);";
	put "			);";
	put "	);";	   
	put ");";
	put "Try(Here:alldat = open(""&ClientOutPath.&OutDataName_sum..sas7bdat"", invisible);";
	put "   Here:alldat << Select Where( Contains(selsub, Column(subname)[]));";
	put "   Current Data Table(Here:alldat);";
	put " );";
	put ");";

	%if %sysfunc(exist(mindist)) %then %do;	
		put "Here:fbox0=Function({},{default local},";
		put "Here:mindist = open(""&ClientOutPath.&OutDataName_min..sas7bdat"",invisible);";
		put "Here:alldist = open(""&ClientOutPath.&OutDataName_all..sas7bdat"",invisible);";
		put "Column(Here:mindist, ""&SITEIhlabel"") << Label(1);";
		put "Column(Here:alldist, ""&SubjectLabel"") << Label(1);";
		put "Column(Here:alldist, ""&SubjectLabel2"") << Label(1);";
		put "Here:allDistgraphs=HListBox(";
		put "		Here:alldist << Graph Builder(";
		put "			Automatic Recalc(1),";
		put "			Auto Stretching( 0 ),";	
		put "			Show Control Panel( 0 ),";
		put "			Show Legend( 0 ),";
		put "			Variables( X( :&SITEIhlabel ), Y( :&trDistance )),";
		put "			Elements(";
		put "				Box Plot(";
		put "					X,";
		put "					Y,";
		put "					Legend( 2 ),";
		put "					Jitter( 1 ),";
		put "					Outliers( 1 ),";
		put "					Box Style( ""Solid"" )";
		put "				),";
		put "			),";
		put "			SendToReport(";
		put "				Dispatch(";
		put "					{},";
		put "					""Graph Builder"",";
		put "					OutlineBox,";
		put "					{Set Title( locale:get(""BetweenSubjectDistancesbySite_Text"") )}";
		put "				),";
		put "				Dispatch(";
		put "					{},";
		put "					""&SITEIhlabel"",";
		put "					ScaleBox,";
		put "					{Show Major Ticks( 0 ), Rotated Labels( ""Automatic"" )}";
		put "				),";
		put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
		put "				Dispatch(";
		put "					{},";
		put "					""Graph Builder"",";
		put "					FrameBox,";
		put "					{Marker Size( 2 ), Transparency( 0.75 )}";
		put "				)";
		put "			)";
		put "		),";
		put "		Here:mindist << Graph Builder(";
		put "			Size( 353, 497 ),";
		put "			Automatic Recalc(1),";
		put "			Auto Stretching( 0 ),";	
		put "			Show Control Panel( 0 ),";
		put "			Show Legend( 0 ),";
		put "			Variables( Y( :&trDistance )),";
		put "			Elements(";
		put "				Box Plot(";
		put "					Y,";
		put "					Legend( 2 ),";
		put "					Jitter( 1 ),";
		put "					Outliers( 1 ),";
		put "					Box Style( ""Outlier"" )";
		put "				),";
		put "				Points( Y,Legend( 3 ), Jitter( 1 ) )";
		put "			),";
		put "			SendToReport(";
		put "				Dispatch(";
		put "					{},";
		put "					""Graph Builder"",";
		put "					OutlineBox,";
		put "					{Set Title( locale:get(""MinimumBetweenSubjectDistancesforEachSite_Text"") )}";
		put "				),";
		put "				Dispatch( {}, """", ScaleBox, {Rotated Labels( ""Automatic"" )} ),";
		put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
		put "				Dispatch(";
		put "					{},";
		put "					""Graph Builder"",";
		put "					FrameBox,";
		put "					{Marker Size( 2 ), Transparency( 0.75 )}";
		put "				)";
		put "			)";
		put "		),";		
		put ");";
		put ");";
		put "InsertInto(DataList,{{""Here:mindist"",""Here:alldist""}});";
		put "InsertInto(TabButtonList, expr(locale:get(""BetweenSubjectDistanceSummary_Text"")));";
		put "InsertInto(TabCodeList,expr(Here:fbox0()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";
		put "InsertInto(Here:CreateReportList,{{Here:allDistgraphs}});";
	%end;
	%let i = 1;
	%let tFlag = 0;
	%do %while(%length(%scan(&SiteList,&i,' ')));
		%let tempsite = %scan(%upcase(&SiteList),&i," ");
		%if ^%index(%bquote(&SitesNoVariability), %bquote(&tempsite)) %then %do;
			put "Here:fbox&tempsite=Function({},{default local},";
			%if %nrbquote(&SYSENCODING) = wlatin1 %then %do;
				put "Here:ddist&tempsite = open(""&ClientOutPath.&&&OutDataName_&tempsite...sas7bdat"",invisible);";
				put "Here:dpair&tempsite = open(""&ClientOutPath.&&&OutDataNameP_&tempsite...sas7bdat"",invisible);";	
			%end;
			%else %do;
				put "Here:ddist&tempsite = open(""&ClientOutPath.&&&OutDataName_&i...sas7bdat"",invisible);";
				put "Here:dpair&tempsite = open(""&ClientOutPath.&&&OutDataNameP_&i...sas7bdat"",invisible);";	
			%end;
			put "Column(Here:dpair&tempsite, ""&SubjectLabel"") << Label(1);";
			put "Column(Here:dpair&tempsite, ""&SubjectLabel2"") << Label(1);";
			put "Here:Dm&tempsite=VListBox(";	   
			%if &tempsite = &MinDistSite %then %do;			
				put "textbox(locale:get(""Thissitehastheminimumbetweensubjectdistance_Text""),<<Set Wrap(900),<< Font Color(5)),"; 
				put "textbox("""",<<Set Wrap(900),<< Font Color(5)),"; 
			%end; 
			put "HListBox(";
			put "		Here:dpair&tempsite << Graph Builder(";
			put "			Size( 353, 497 ),";
			put "			Automatic Recalc(1),";
			put "			Auto Stretching( 0 ),";      
			put "			Show Control Panel( 0 ),";
			put "			Show Legend( 0 ),";
			put "			Variables( Y( :&trDistance )),";
			put "			Elements(";
			put "				Box Plot(";
			put "					Y,";
			put "					Legend( 2 ),";
			put "					Jitter( 1 ),";
			put "					Outliers( 1 ),";
			put "					Box Style( ""Outlier"" )";
			put "				),";
			put "				Points( Y, Legend( 3 ), Jitter( 1 ) )";
			put "			),";
			put "			SendToReport(";
			put "				Dispatch(";
			put "					{},";
			put "					""Graph Builder"",";
			put "					OutlineBox,";
			put "					{Set Title( locale:get(""BetweenSubjectDistances_Text"") )}";
			put "				),";
			put "				Dispatch( {}, """", ScaleBox, {Rotated Labels( ""Automatic"" )} ),";
			put "				Dispatch( {}, ""graph title"", TextEditBox, {Set Text( """" )} ),";
			put "				Dispatch(";
			put "					{},";
			put "					""Graph Builder"",";
			put "					FrameBox,";
			put "					{Marker Size( 2 ), Transparency( 0.75 )}";
			put "				)";
			put "			)";
			put "		),";
			put "		Here:ddist&tempsite << Hierarchical Cluster(";
			put "			Automatic Recalc(1),";
			put "       Hide Distance Messages(1),";			
			put "			Y(&&SiteClusterVars&tempsite),";
			put "			Label( :&SUBJECTLabel ),";
			put "			Method( ""&HierClustMethod"" ),";
			put "			Standardize( 0 ),";
			put "			Distance Matrix( 1 ),";
			put "			Color Clusters( 1 ),";
			put "			Distance Graph( 0 ),";
			put "			Legend( 1 ),";
			put "			Dendrogram Scale( ""Distance Scale"" ),";
			put "			Two Way Clustering,";
			put "			Color Map( {"""", {{255, 0, 0}, {255, 255, 255}}} ),";
			put "			SendToReport(";
			put "			Dispatch(";
			put "				{""Dendrogram""},";
			put "				""Clust Dendro"",";
			put "				FrameBox,";
			put "				{Frame Size( 45, 352 )}";
			put "			),";
			put "			Dispatch( {""Dendrogram""}, """", CellPlotBox( 2 ), {Frame Size( 349, 352 ), Color Theme({"""", {{255, 0, 0}, {255, 255, 255}}})} ),";
			put "			Dispatch(";
			put "				{""Dendrogram""},";
			put "				""Clust Dendro"",";
			put "				FrameBox( 2 ),";
			put "				{Frame Size( 200, 352 )}";
			put "			),";
			put "			Dispatch(";
			put "				{""Dendrogram""},";
			put "				""Clust Dendro"",";
			put "				FrameBox( 3 ),";
			put "				{Frame Size( 349, 155 )}";
			put "			),";
			put "			Dispatch(";
			put "				{""Dendrogram""},";
			put "				""Clust Dendro"",";
			put "				FrameBox( 4 ),";
			put "				{Frame Size( 349, 59 )}";
			put "			))";				
			put "		);";
			put ")));";
			put "InsertInto(CreateReportList,{{Here:fbox&tempsite}});";
			put "InsertInto(DataList,{{""Here:ddist&tempsite"", ""Here:dpair&tempsite""}});";
			put "InsertInto(TabButtonList, expr(locale:fill(locale:get(""SiteXDistanceMatrix_Text""),{""&tempsite""})));";
			put "InsertInto(TabCodeList,expr(Here:fbox&tempsite()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			%if &tempsite = &MinDistSite %then %do;
				put "InsertInto(TabOpenList,1);";
			%end;
			%else %do;
				put "InsertInto(TabOpenList,0);";
				%let tFlag = 1;
			%end;
		%end;
		%else %do;
			put "Here:fbox&tempsite=Function({},{default local},";
			put "VListBox(TextBox(locale:get(""AllVarsAvForAnHadNoVarWiThisSiteThisImpMsg_Text""),<<Set Wrap(900),<< Font Color(5)));"; 
		   put ");";
			put "InsertInto(DataList,{{""""}});";
			put "InsertInto(TabButtonList, expr(locale:fill(locale:get(""SiteXDistanceMatrix_Text""),{""&tempsite""})));";
			put "InsertInto(TabCodeList,expr(Here:fbox&tempsite()));";
			put "InsertInto(TabAfterCodeList, "" "");";
			put "InsertInto(TabOpenList,1);";
		%end;
		%let i = %eval(&i+1);
   %end;
	%if &tFlag=1 %then %do;
		put "Here:TabsSectionFlag = 1;";
	%end;
  
	put "Here:ppdd =  Function({},{},";
	put "Here:Select();";
	put "Include(""$CLINICAL_HOME\ProcessLibrary\PatientProfiler.jsl"",<< New Context,<< ""Names Default To Here""); Close(Here:alldat, nosave););";
	put "Here:spdd =  Function({}, {},";
	put "Here:Select();";
	put "Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""); Close(Here:alldat, nosave););";
	put "Include(""$CLINICAL_HOME\JSLFiles\Fraud Functions.jsl"");";
	put "Here:ActionButtonList= {{expr(locale:get(""ProfileSubjects_Text"")),expr(locale:get(""ShowSubjects_Text"")),";
	put "   expr(locale:get(""ShowRowsinHeatMap_Text"")),expr(locale:get(""SubsetClusteringRows_Text"")),expr(locale:get(""RevertClustering_Text""))}};";
	put "Here:ActionCodeList= {{Here:ppdd(),Here:spdd(),Here:ShowRow(), Here:SubsetClustering(),Here:RevertClustering()}};";
	put "Here:ActionButtonDesc = {expr(locale:get(""SelectRecordsthenClick_Text""))};";

%let includeds = %substr(%quote(&includeds), 1, %eval(%length(&includeds) - 1));
%PackageInsertFile(jsl code from &ProcessName, &OutPath, &ClusterSubjectsScriptName, "&includeds", yes);
%PackageInsertDataset(%nrbquote(%LocaleFill(%nrbquote(%LocaleGet(DataFromX_Text)),
	%nrbquote(%LocaleGet(&ProcessName._Label)))), outlib,  &OutDataName_sum, no);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend ClusterSubjectsStudySite;
%ClusterSubjectsStudySite;

************************************;
************************************;
************************************;