/*
JMP Clinical Analytical Process

Name: Duplicate Records
Authors: Richard C. Zink, SAS Institute Inc.

Description: 
	This process identifies sets of records that have identical values on more than one occassion within a subject or between subjects within a study site.
*/

/*
-----------------------------------------------------------
Copyright (C) 2012 SAS Institute, Inc. All rights reserved. 

Notice: 
The above copyright notice and this notice must appear in
any whole or partial copy of this code and any related
documentation. Permission to use, copy, and modify the
source code contained in this file is hereby granted, but
is limited to customers of SAS with a valid license for the
SAS software product with which this file was provided. 
Except as expressly set forth herein, the terms of such
license apply.
-----------------------------------------------------------
*/

*ProcessVariables;

*options mprint mlogic symbolgen;

%global StudyName InPath AInPath SubjectFilter TrtVar TrtUse MacroPath OutName DuplicateRecordsScript DuplicateRecords_OutData OutPath 
        ServerOutPath PopulationSet ResType IgnoreSubFl IgnoreCovFl WhereStmt WhereForReport;

*ProcessBody;

%macro DuplicateRecords;

*initialization and argument checking;

%let ProcessName = DuplicateRecords; 

%include "&MacroPath./UtilityMacros.sas" / nosource;
%include "&MacroPath./CDISCParms.sas";
%if &exiterror %then %goto exit;
%let WhereForReport=%nrbquote(&WhereStmt);
/*%let domainlist = &findingsdomains;*/
%let domainlist = &findingsdomains;
%let suffix = LOC METHOD POS SPEC TPT;

%if ^%length(&findingsdomains) %then %do;
	%put ERROR: No Findings domains are available for the current study.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), ARM, ' ')) %then %do;
	%put ERROR: ARM is not available in either DM, ADDM or ADSL.;
	%let exiterror = 1;
	%goto exit;
%end;

%if ^%sysfunc(findw(%upcase(&ADSLVARS), SITEID, ' ')) %then %do;
	%put ERROR: SITEID is not available in either DM, ADDM or ADSL.;
	%let exiterror = 1;
	%goto exit;
%end;

%if %index(%upcase(&in_adsl), ADSL) and %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY, ' ')) = 0 and %sysfunc(findw(%upcase(&DMVARS), COUNTRY, ' ')) > 0 %then %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm);
		by usubjid;
	run;

	proc sort data = &in_dm out = dm;
		by usubjid;
	run;

	data subject;
		merge subject(in = ina) dm(keep = usubjid country);
		by usubjid;
		if ina;
	run;
%end;
%else %if %sysfunc(findw(%upcase(&ADSLVARS), COUNTRY, ' ')) %then %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm country);
		by usubjid;
	run;
%end;
%else %do;
	proc sort data = &in_adsl out = subject(keep = usubjid siteid arm);
		by usubjid;
	run;
	
	data subject;
	   length Country $ 12;
	   set subject;
	   country = "Not Provided";
	   label country = "Country";
	run;
%end;

*** create by lists for each domain ***;
%let i = 1;
%do %while(%length(%scan(&domainlist,&i,' ')));
	%let temp = %scan(&domainlist,&i,' ');
   %let subbyvar_&temp =;
	%let byvar_&temp = ;
	%let var_&temp = ;

   %if %sysfunc(indexw(&&&temp.VARS, VISITNUM, ' ')) %then %let byvar_&temp = &&&byvar_&temp VISITNUM;
	%let j = 1; 
   %do %while(%length(%scan(&suffix,&j,' ')));
      %let stemp = %scan(&suffix,&j,' ');
      %if %sysfunc(indexw(&&&temp.VARS, &temp&stemp, ' ')) %then %let byvar_&temp = &&&byvar_&temp &temp&stemp;
      %let j = %eval(&j+1);
   %end;   

   %if %upcase(&ResType) = NUMERIC RESULTS IN STANDARD UNITS and %sysfunc(indexw(&&&temp.VARS, &temp.STRESN, ' ')) %then %let var_&temp = &&&var_&temp &temp.STRESN;
   %else %if %upcase(&ResType) = CHARACTER RESULTS IN STANDARD FORMAT and %sysfunc(indexw(&&&temp.VARS, &temp.STRESC, ' ')) %then %let var_&temp = &&&var_&temp &temp.STRESC;
   %else %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS and %sysfunc(indexw(&&&temp.VARS, &temp.ORRES, ' ')) %then %let var_&temp = &&&var_&temp &temp.ORRES;
	%let i = %eval(&i+1);
	%let subbyvar_&temp = &&&byvar_&temp;
	
%end;

%if %upcase(&ResType) = NUMERIC RESULTS IN STANDARD UNITS %then %let varsuff = STRESN;
%else %if %upcase(&ResType) = CHARACTER RESULTS IN STANDARD FORMAT %then %let varsuff = STRESC;
%else %if %upcase(&ResType) = RESULTS IN ORIGINAL UNITS %then %let varsuff = ORRES;


%let flag = 1;
%let includeds = ;
%let i = 1;
%do %while(%length(%scan(&domainlist,&i,' ')));
	%let temp = %scan(&domainlist,&i,' ');
	%let tind = %sysfunc(left(%sysfunc(countw(%sysfunc(compbl("&&&var_&temp")), " "))));

   %if %length(&&&var_&temp) %then %do;   
      data &temp(keep = usubjid &temp.seq &temp.dtc &temp.test &temp.testcd &&&byvar_&temp &&&var_&temp);
         set &&in_&temp(where = (not missing (&&&var_&temp)));         
      run;
		
		%let _tid_ = %sysfunc(open(&temp));
		%let vn = %sysfunc(varnum(&_tid_,&temp.testcd));
		%let CDLen = %sysfunc(varlength(&_tid_,&vn));
		%let _tid_ = %sysfunc(close(&_tid_));
       
      %let CDLen = %sysfunc(max(&CDLen, 8));
    		
		data &temp;
		   length &temp.testcd $ &CDlen;
		   set &temp;
		   if &temp.testcd = "" then &temp.testcd = "NOTGIVEN";		
		run;

		proc sort data = &temp out = &temp;
			by usubjid &&&byvar_&temp &temp.testcd;
		run;

		data &temp;
			set &temp;
			%TIME5(var=&temp.dtc,return=time);
			%DATE9(var=&temp.dtc,return=date);
			drop &temp.dtc;
			label date = "Date" time = "Time";
		run;

      *** Test for duplicates within by variables ***;
		data &temp.test;
		   set &temp;
		   by usubjid &&&byvar_&temp &temp.testcd;
		   if first.&temp.testcd ne 1 or last.&temp.testcd ne 1;
		run;
	   %NObsVars(&temp.test);
	   
	   *** if dups present, add date and time to by variables ***;
	   %if &nobs %then %do;	   
	      %let byvar_&temp = date time &&&byvar_&temp;
	   	proc sort data = &temp out = &temp;
			   by usubjid &&&byvar_&temp &temp.testcd;
			run;		
      %end;         

		data &temp.test;
		   set &temp;
		   by usubjid &&&byvar_&temp &temp.testcd;
		   if first.&temp.testcd ne 1 or last.&temp.testcd ne 1;
		run;
		
	   %NObsVars(&temp.test);

	   *** if dups still present, remove them ***;
	   %if &nobs %then %do;	   
			data &temp.test;
				set &temp;
				by usubjid &&&byvar_&temp &temp.testcd;
				if not first.&temp.testcd;
			run;

			proc sort data = &temp.test(keep = usubjid &temp.seq);
				by usubjid &temp.seq;
			run;

			proc sort data = &temp;
				by usubjid &temp.seq;
			run;

			data &temp;
				merge &temp(in = ina) &temp.test(in = inb);
				by usubjid &temp.seq;
				if ina and not inb;
			run;
			
			proc sort data = &temp out = &temp;
				by usubjid &&&byvar_&temp &temp.testcd;
			run;		
      %end;

      %let codes_&temp = ;
      %let numcodes_&temp = 0;
      proc sql noprint;      
         select distinct &temp.testcd
		   into :codes_&temp separated by ' '
		   from &temp
		   order by &temp.testcd;
		   %let numcodes_&temp = &sqlobs;
		quit;

      proc transpose data = &temp out = &temp(drop = _name_ _label_);
         by usubjid &&&byvar_&temp;
         var &&&var_&temp;
         id &temp.testcd;
         idlabel &temp.test;
      run;

      data &temp;
         merge subject(in=ina keep = usubjid siteid) &temp(in=inb);
         by usubjid;
         if ina and inb;
      run;

      proc sort data = &temp;
         by siteid %if &IgnoreCovFl = Yes %then %do; &&&subbyvar_&temp %end; &&&codes_&temp usubjid;
      run;

      data &temp;
         set &temp;
         by siteid %if &IgnoreCovFl = Yes %then %do; &&&subbyvar_&temp %end; &&&codes_&temp;
         if first.%scan(&&&codes_&temp, -1, ' ') ne 1 or last.%scan(&&&codes_&temp, -1, ' ') ne 1;         
      run;
      
      data &temp;
         set &temp;
         by siteid %if &IgnoreCovFl = Yes %then %do; &&&subbyvar_&temp %end; &&&codes_&temp;
         retain duplicateid;
         if _n_ = 1 then duplicateid = 1;
         else if first.%scan(&&&codes_&temp, -1, ' ') then duplicateid = duplicateid + 1;
      run;
      
      %if &IgnoreSubFl = Yes %then %do;
         *** drops any duplications of usubjid within a duplicate record id ***;
			data &temp;
				set &temp;
				by duplicateid usubjid;
				if first.usubjid;         
			run;
			
			*** drops any duplicate record id with only one record (only existed cause of a duplicate within subject) ***;
			data &temp(drop = duplicateid);
				set &temp;
				by duplicateid;
				if first.duplicateid = 1 and last.duplicateid = 1 then delete;         
			run;			

			*** Regenerate Duplicate ID ***;			
			data &temp;
				set &temp;
				by siteid %if &IgnoreCovFl = Yes %then %do; &&&subbyvar_&temp %end; &&&codes_&temp;
				retain duplicateid;
				if _n_ = 1 then duplicateid = 1;
				else if first.%scan(&&&codes_&temp, -1, ' ') then duplicateid = duplicateid + 1;
			run;	
      %end;            

      *** Merge in with ADSL for Filter ***;
		proc sort data = &temp;
			by usubjid;
		run;

		data &temp;
			merge subject(in = ina) &temp(in = inb);
			by usubjid;
			if ina and inb;
		run;	   

		proc sort data = &temp;
			by duplicateid usubjid;
		run;

		*** drops any duplicate record id with only one record (only existed cause of a duplicate within subject) ***;
		data &temp(drop = duplicateid);
			set &temp;
			by duplicateid;
			if first.duplicateid = 1 and last.duplicateid = 1 then delete;         
		run;			

		proc sort data = &temp;
			by siteid %if &IgnoreCovFl = Yes %then %do; &&&subbyvar_&temp %end; &&&codes_&temp usubjid;
		run;

		*** Regenerate Duplicate ID ***;			
		data &temp;
			set &temp;
			by siteid %if &IgnoreCovFl = Yes %then %do; &&&subbyvar_&temp %end; &&&codes_&temp;
			retain duplicateid;
			if _n_ = 1 then duplicateid = 1;
			else if first.%scan(&&&codes_&temp, -1, ' ') then duplicateid = duplicateid + 1;
			label duplicateid = "Duplicate ID";			
		run;	

	%end;
	%else %do;
      data &temp;
      	if _n_ = 1 then delete;
      run;	   
	%end;
	
	%NObsVars(&temp);
	%let nobs_&temp = &nobs;
	%if &nobs %then %do;
	   %let DuplicateRecords_OutData_&temp = &DuplicateRecords_OutData;	   
		%PrepOutDS(DuplicateRecords_OutData_&temp,OutDataName_&temp, DuplicateRecords, _&temp);
		%if &flag %then %do;
		   %let includeds = &&&&OutDataName_&temp...sas7bdat;
		   %let flag = 0;
		%end;
		%else %let includeds = &includeds,&&&&OutDataName_&temp...sas7bdat;
		data &&&DuplicateRecords_OutData_&temp;
		   set &temp;
		run;	   
	%end;
	
	proc datasets nolist;
		delete &temp;
	quit;

	%let i = %eval(&i+1);	
%end;

%let ARMLabel = ARM;
%CheckLabel(subject,ARMLabel);
%let SUBJECTLabel = USUBJID;
%CheckLabel(subject,SUBJECTLabel);
%let COUNTRYLabel = COUNTRY;
%CheckLabel(subject,COUNTRYLabel);
%let SITEIhlabel = SITEID;
%CheckLabel(subject,SITEIhlabel);

* create JSL file;
%PrepOutFile(DuplicateRecordsScript,DuplicateRecordsScriptName,&StudyName &ProcessName Script.jsl);
%global _mainjslfile_ FullAPName;
%let _mainjslfile_ = &DuplicateRecordsScript;
%let FullAPName=%str(Duplicate Records);

* put JSL statements;
data _null_;
   file "&DuplicateRecordsScript" %if %nrbquote(&SYSENCODING) ne wlatin1 %then encoding="utf-8";;
	put "//!";
	put "Names Default to Here(1);";
	put "Here:DataList = {};";
	put "Here:TabButtonList = {};";
	put "Here:TabCodeList = {};";
	put "Here:TabAfterCodeList = {};";
	put "Here:TabOpenList = {};";
	put "Here:CreateReportList = {};";
	
	put "try(Here:SubjectFilter=""\[&SubjectFilter]\"");";
	put "try(Here:WhereState=""\[&WhereForReport]\"");";
	put "try(Here:PopulationSet=""\[&PopulationSet]\"");";
	put "try(Here:WhereFilters=""Additional Filter to include Subjects: "" || Here:WhereState);";	
	
	

   %let i = 1;
	%do %while(%length(%scan(&domainlist,&i,' ')));
		%let temp = %scan(&domainlist,&i,' ');
		%let templab = %bquote(%scan(&findingsdomainslabel,&i,~));
		
		put "Here:f&temp=Function({},{default local},";
		put "	 if(""&templab"" == ""ECG Test Results"", trtemplab = locale:get(""ECGTestResults_Text""),";
		put	"		""&templab"" == ""Laboratory Test Results"", trtemplab = locale:get(""LaboratoryTestResults_Text""),";
		put "		""&templab"" == ""Vital Signs"", trtemplab = locale:get(""VitalSigns_Text""),";
		put "		trtemplab = ""&templab"");";
		put "	 if(""%lowcase(&ResType)"" == ""numeric results in standard units"", trlcResType = locale:get(""numericresultsinstandardunits_Text""),";
		put "		""%lowcase(&ResType)"" == ""character results in standard format"", trlcResType = locale:get(""characterresultsinstandardformat_Text""),";
		put "		""%lowcase(&ResType)"" == ""results in original units"", trlcResType = locale:get(""resultsinoriginalunits_Text""),";
		put "		trlcResType = ""%lowcase(&ResType)"");";
		put "	 trlcrttv = trlcResType || "" "" || ""(&temp&varsuff)"";";
		%if &&&nobs_&temp %then %do;
		
			put "   If(IsEmpty(Here:d&temp),";
			put "      Here:d&temp=open(""&ClientOutPath.&&&OutDataName_&temp...sas7bdat"",invisible);";
			put "      Column(""&SubjectLabel"") << Label(1);";
			put "   );";
			put " if (N Rows(Here:d&temp) != 0,";  
			put "	   VListBox(Here:dups&temp=textbox(locale:fill(locale:get(""DuplicaterecordsarebasedonX_Text""),{trlcResType}),<<Font Color(5), <<Set Wrap(750)),";
			put "      HListBox(";
			put "	       Here:dp&temp = Distribution(";
			put "	           Automatic Recalc(1),";
			put "	            Nominal Distribution(Column(:Name(""Duplicate ID"")), ShowCounts(1)),";
			put "	            Nominal Distribution(Column(:Name(""&SUBJECTLabel"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
			put "	            Nominal Distribution(Column(:Name(""&COUNTRYLabel"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
			put "	            Nominal Distribution(Column(:Name(""&SITEIhlabel"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
			put "	            Nominal Distribution(Column(:Name(""&ARMlabel"")), ShowCounts(1), Order By( ""Count Ascending"" )),";
			put "	           SendToReport(";
			put "	              Dispatch( {""Duplicate ID""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
			put "	              Dispatch( {""&SUBJECTLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
			put "	              Dispatch( {""&COUNTRYLabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";
			put "	              Dispatch( {""&SITEIhlabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";		
			put "	              Dispatch( {""&ARMlabel""}, ""Frequencies"", OutlineBox, Close( 1 ) ),";		
			put "	           )";
			put "	       ),";
			%let RepExpr&temp=%nrbquote(Here:dups&temp,Here:dp&temp);
			put "Here:Filter=Current Data Table() << Data Filter(<<Conditional(1),";
			put "	           Add Filter(";
			put "	              Columns(:Name(""&COUNTRYLabel""), :Name(""&SITEIhlabel""))"; 
			put "	           ),";
			put "	  		     Mode( Show(1), Include(1) )";
			put "	        )";
			put "	     )"; 
			put "	   )";
			put "	   , //else ";
			put "    Here:dups=textbox(locale:fill(locale:get(""NoSubjsHadDupRecswiXdomainforX_Text""),{trtemplab, trlcrttv}),<<Font Color(5), <<Set Wrap(750));";
			put "    );";
		%end;
		%else %if ^&&&nobs_&temp and %length(&&&var_&temp) %then %do;
			put "    Here:TB&temp=textbox(locale:fill(locale:get(""NoSubjsHadDupRecswiXdomainforX_Text""),{trtemplab, trlcrttv}),<<Font Color(5), <<Set Wrap(750));";
			%let RepExpr&temp=%nrbquote(Here:TB&temp);
		%end;
		%else %if ^&&&nobs_&temp and ^%length(&&&var_&temp) %then %do;
			put "    Here:TB&temp=textbox(locale:fill(locale:get(""TheXDomDoesNotHaveVarforX_Text""), {trtemplab, trlcrttv}),<<Font Color(5), <<Set Wrap(750));";
			%let RepExpr&temp=%nrbquote(Here:TB&temp);
		%end;
		put "	   ); //Close function";	
		
		%if &&&nobs_&temp %then %do;
		   put "InsertInto(DataList,{{""Here:d&temp""}});";
		%end;
		%else %do;
		   put "InsertInto(DataList,{{""""}});";		
		%end;
		put "TabToDisplay = ""&templab"";";
		put "if (TabToDisplay == ""ECG Test Results"", TabToDisplay = locale:get(""ECGTestResults_Text""););"; 
		put "if (TabToDisplay == ""Laboratory Test Results"", TabToDisplay = locale:get(""LaboratoryTestResults_Text""););"; 
		put "if (TabToDisplay == ""Vital Signs"", TabToDisplay = locale:get(""VitalSigns_Text""););"; 
		put "InsertInto(TabButtonList, TabToDisplay);";
		put "InsertInto(TabCodeList,expr(Here:f&temp()));";
		put "InsertInto(TabAfterCodeList, "" "");";
		put "InsertInto(TabOpenList,1);";
		put "InsertInto(Here:CreateReportList,{{&&RepExpr&temp}});";
	   %let i = %eval(&i+1);
	%end;
	
	put "Here:spdd =  Function({}, {}, Include(""$CLINICAL_HOME\ProcessLibrary\ShowSubjects.jsl""));";
	put "Here:dpdd =  Function({}, {}, Include(""$CLINICAL_HOME\JSLFiles\ShowDuplicates.jsl""));";	
	put "Here:ActionButtonList= {{expr(locale:get(""ShowSubjects_Text"")),expr(locale:get(""ShowDuplicates_Text""))}};";
	put "Here:ActionCodeList= {{Here:spdd(),Here:dpdd()}};";
	put "Here:ActionButtonDesc = {expr(locale:get(""SelectRecordsthenClick_Text""))};";

%PackageInsertFile(jsl code from &ProcessName, &OutPath, &DuplicateRecordsScriptName, "&includeds", yes);

%exit:;
%ExitSDSProcess(&OutPath, &ProcessName, &syserr, &exiterror, &clockrunning);

%mend DuplicateRecords;
%DuplicateRecords;

************************************;
************************************;
************************************;
